@@ -6,9 +6,6 @@ import numpy as np
 from batchflow import Dataset, ImagesBatch
 
 
-from .config_pass_test import single_config, multi_config, model_and_config
-
-
 @pytest.fixture()
 def model_setup_images_clf():
     """ Pytest fixture to generate fake dataset and model config for image classification
@@ -40,9 +37,7 @@ def model_setup_images_clf():
                           batch_class=ImagesBatch,
                           preloaded=data)
 
-        model_config = {'inputs/images/shape': image_shape,
-                        'inputs/labels/classes': num_classes,
-                        'initial_block/inputs': 'images'}
+        model_config = {'classes': 10}
         return dataset, model_config
 
     return _model_setup
