@@ -2,10 +2,12 @@
 import os
 import re
 import json
+import warnings
 
 import numpy as np
 
-# Additionally imports 'requests`, 'ipykernel`, `notebook`, `nbconvert`, `pylint` and `nvidia_smi`, if needed
+# Additionally imports 'requests`, 'ipykernel`, `jupyter_server`, `nbconvert`, `pylint`,
+#                      'nbconvert', 'IPython' and `nvidia_smi`, if needed
 
 
 def in_notebook():
@@ -31,19 +33,25 @@ def get_notebook_path():
 
     import requests
     import ipykernel
-    from notebook.notebookapp import list_running_servers
 
-    kernel_id = re.search('kernel-(.*).json',
-                          ipykernel.connect.get_connection_file()).group(1)
-    servers = list_running_servers()
+    # Id of the current running kernel: a string uid
+    kernel_id = re.search('kernel-(.*).json', ipykernel.connect.get_connection_file()).group(1)
+
+    # Get running servers for both JupyterLab v2.# and v3.#
+    from notebook.notebookapp import list_running_servers as list_running_servers_v2
+    from jupyter_server.serverapp import list_running_servers as list_running_servers_v3
+    servers = list(list_running_servers_v2()) + list(list_running_servers_v3())
+
     for server in servers:
+        root_dir = server.get('root_dir') or server.get('notebook_dir')
         response = requests.get(requests.compat.urljoin(server['url'], 'api/sessions'),
                                 params={'token': server.get('token', '')})
+
         for params in json.loads(response.text):
             if params['kernel']['id'] == kernel_id:
                 relative_path = params['notebook']['path']
-                return os.path.join(server['notebook_dir'], relative_path)
-    return None
+                return os.path.join(root_dir, relative_path)
+    raise ValueError(f'Unable to find kernel `{kernel_id}` in {len(servers)} servers!')
 
 def get_notebook_name():
     """ Return the title of the current Jupyter notebook without base directory and extension,
@@ -57,8 +65,8 @@ def get_notebook_name():
     return os.path.splitext(get_notebook_path())[0].split('/')[-1]
 
 
-def pylint_notebook(path=None, options='', printer=print, ignore_comments=True, ignore_codes=None,
-                    keep_script=False, return_report=False):
+def pylint_notebook(path=None, options='', printer=print, ignore_comments=True, ignore_codes=tuple(),
+                    use_pylintrc=True, keep_script=False, return_report=False):
     """ Run pylint on entire Jupyter notebook.
     Under the hood, the notebook is converted to regular `.py` script,
     special IPython commands like magics removed, and then pylint is executed.
@@ -78,6 +86,8 @@ def pylint_notebook(path=None, options='', printer=print, ignore_comments=True,
     ignore_codes : sequence
         Pylint errors to ignore.
         By default, `invalid-name`, `import-error` and `wrong-import-position` are disabled.
+    use_pylintrc : bool
+        Whether to use the `BatchFlow` pylint configuration.
     keep_script : bool
         Whether to keep temporal `.py` file after command execution.
     return_report : bool
@@ -90,9 +100,27 @@ def pylint_notebook(path=None, options='', printer=print, ignore_comments=True,
     from nbconvert import PythonExporter
     from pylint import epylint as lint
 
+    # Parse parameters
     path = path or get_notebook_path()
     options = options if options.startswith(' ') else ' ' + options
-    ignore_codes = ignore_codes or ['invalid-name', 'import-error', 'wrong-import-position']
+    ignore_codes = set(ignore_codes)
+    ignore_codes.update({'import-error', 'wrong-import-position', 'invalid-name',
+                         'unnecessary-semicolon', 'trailing-whitespace', 'trailing-newlines'})
+
+    # Try to add `pylintrc` configuration file to options
+    if use_pylintrc and 'rcfile' not in options:
+        # Locate the batchflow pylintrcfile
+        # The loop converts the __file__ path, which is a combination of absolute and relative path, to an absolute
+        pylintcrc_path = []
+        for item in __file__.split('/')[:-2]:
+            if item != '..':
+                pylintcrc_path.append(item)
+            else:
+                pylintcrc_path.pop(-1)
+        pylintcrc_path = '/' + os.path.join(*pylintcrc_path, 'pylintrc')
+
+        if os.path.exists(pylintcrc_path):
+            options += f' --rcfile {pylintcrc_path}'
 
     # Convert the notebook contents to raw string without outputs
     code, _ = PythonExporter().from_filename(path)
@@ -187,19 +215,22 @@ def pylint_notebook(path=None, options='', printer=print, ignore_comments=True,
     return 0
 
 
-def get_available_gpus(n=1, min_free_memory=0.9, max_processes=2, verbose=False):
+def get_available_gpus(n=1, min_free_memory=0.9, max_processes=2, verbose=False, raise_error=False):
     """ Select `n` gpus from available and free devices.
 
     Parameters
     ----------
-    n : int
-        Number of devices to select.
+    n : int, str
+        If `max`, then use maximum number of available devices.
+        If int, then number of devices to select.
     min_free_memory : float
         Minimum percentage of free memory on a device to consider it free.
     max_processes : int
         Maximum amount of computed processes on a device to consider it free.
     verbose : bool
         Whether to show individual device information.
+    raise_error : bool
+        Whether to raise an exception if not enough devices are available.
 
     Returns
     -------
@@ -230,18 +261,40 @@ def get_available_gpus(n=1, min_free_memory=0.9, max_processes=2, verbose=False)
             print(f'Device {i} | Free memory: {fraction_free:4.2f} | '
                   f'Number of running processes: {num_processes:>2} | Free: {consider_available}')
 
+    if isinstance(n, str) and n.startswith('max'):
+        n = len(available_devices)
+
     if len(available_devices) < n:
-        raise ValueError(f'Not enough free devices: requested {n}, found {len(available_devices)}')
+        msg = f'Not enough free devices: requested {n}, found {len(available_devices)}'
+        if raise_error:
+            raise ValueError(msg)
+        warnings.warn(msg, RuntimeWarning)
+
     available_devices = np.array(available_devices)[np.argsort(memory_usage)[::-1]]
     return sorted(available_devices[:n])
 
-def set_gpus(n=1, min_free_memory=0.9, max_processes=2, verbose=False):
+def get_gpu_free_memory(index):
+    """ Get free memory of the gpu"""
+    try:
+        import nvidia_smi
+    except ImportError as exception:
+        raise ImportError('Install Python interface for nvidia_smi') from exception
+
+    nvidia_smi.nvmlInit()
+    nvidia_smi.nvmlDeviceGetCount()
+    handle = nvidia_smi.nvmlDeviceGetHandleByIndex(index)
+    info = nvidia_smi.nvmlDeviceGetMemoryInfo(handle)
+
+    return info.free / info.total
+
+def set_gpus(n=1, min_free_memory=0.9, max_processes=2, verbose=False, raise_error=False):
     """ Set the `CUDA_VISIBLE_DEVICES` variable to `n` available devices.
 
     Parameters
     ----------
-    n : int
-        Number of devices to select.
+    n : int, str
+        If `max`, then use maximum number of available devices.
+        If int, then number of devices to select.
     min_free_memory : float
         Minimum percentage of free memory on a device to consider it free.
     max_processes : int
@@ -251,11 +304,16 @@ def set_gpus(n=1, min_free_memory=0.9, max_processes=2, verbose=False):
         If 0 or False, then no information is displayed.
         If 1 or True, then display the value assigned to `CUDA_VISIBLE_DEVICES` variable.
         If 2, then display memory and process information for each device.
+    raise_error : bool
+        Whether to raise an exception if not enough devices are available.
     """
     if 'CUDA_VISIBLE_DEVICES' in os.environ.keys():
-        raise ValueError(f'`CUDA_VISIBLE_DEVICES` is already set to "{os.environ["CUDA_VISIBLE_DEVICES"]}"!')
+        str_devices = os.environ["CUDA_VISIBLE_DEVICES"]
+        warnings.warn(f'`CUDA_VISIBLE_DEVICES` is already set to "{str_devices}"!')
+        return [int(d) for d in str_devices.split(',')]
 
-    devices = get_available_gpus(n=n, min_free_memory=min_free_memory, max_processes=max_processes, verbose=verbose==2)
+    devices = get_available_gpus(n=n, min_free_memory=min_free_memory, max_processes=max_processes,
+                                 verbose=(verbose==2), raise_error=raise_error)
     str_devices = ','.join(str(i) for i in devices)
     os.environ['CUDA_VISIBLE_DEVICES'] = str_devices
 
